services.factory('LocalNotificationService', ['$rootScope', '$translate', function ($rootScope, $translate) {

    var NTYPE = {
        USAGE: "usage",
        CHALLENGE: "challenge",
        RACE: "race",
    };

    function ready() {
        if (typeof (cordova) === "undefined") { return false };
        var ready = cordova.plugins && cordova.plugins.notification && cordova.plugins.notification.local;
        if (!ready) {
            //alert("Local notifications not ready!");
        }
        return ready;
    }

    function getPrefix(type) {
        switch (type) {
            case NTYPE.USAGE:
                return 100000;
            case NTYPE.CHALLENGE:
                return 200000;
            case NTYPE.RACE:
                return 300000;
            default:
                return 0;
        }
    }

    function getUniqueId(type, id) {
        return getPrefix(type) + id;
    }

    function emitEvent(notification) {
        if (notification.data && !notification.data.type) {
            notification.data = JSON.parse(notification.data);
        }
        $rootScope.$broadcast("notification:local", notification);
    }

    function getCurrentSeason() {
        var d = new Date();
        var m = d.getMonth();
        var isSeptember = m >= 8;

        // if it's at least september then the current season starts this year, else last year
        var startYear = isSeptember ? d.getFullYear() : d.getFullYear() - 1;
        // if it's at least september then the current season ends next year, else this year
        var endYear = isSeptember ? d.getFullYear() + 1 : d.getFullYear();
        return {
            // month is 0 based, so this is October 20th
            start: new Date(startYear, 9, 20),
            // month is 0 based, so this is March 20th
            end: new Date(endYear, 2, 20)
        };
    }

    function daydiff(first, second) {
        // 1000 ms * 60 seconds * 60 mins * 24 hours
        // this isn't an exact solution, but it's close enough for our purposes
        return (second - first) / (1000 * 60 * 60 * 24);
    }

    //----Private----
    var
        _init = function () {
            if (!ready()) return;

            cordova.plugins.notification.local.on("click", function (notification) {
                emitEvent(notification);
            });

            // update the usage notifications as part of the init
            _updateUsageNotifications();
        },
        _schedule = function (type, notifications) {
            if (notifications && notifications.length > 0) {
                if (!ready()) return;

                var localNotifications = notifications.map(function (notification) {
                    return {
                        id: getUniqueId(type, notification.id),
                        title: notification.title,
                        text: notification.text,
                        at: notification.at || new Date(),
                        icon: notification.icon || "file://resources/icon.png",
                        smallicon: notification.smallicon || "file://resources/icon.png",
                        sound: notification.sound || deviceIsAndroid ? 'file://resources/SecondGlance.mp3' : 'file://resources/SecondGlance.m4r',
                        data: { "type": type, "id": notification.id }
                    };
                });
                cordova.plugins.notification.local.schedule(localNotifications);
            }
        },
        _cancel = function (type, id) {
            if (!ready()) return;

            var uid = getUniqueId(type, id);
            cordova.plugins.notification.local.isPresent(uid, function (present) {
                if (present) {
                    cordova.plugins.notification.local.cancel(uid, function () {
                        // nothing to do here
                    });
                }
            });
        },
        _updateUsageNotifications = function () {
            var seasonEnd = getCurrentSeason().end;
            var seasonStart = getCurrentSeason().start;
            var days = seasonEnd - seasonStart;
            var interval = 30; // number of days between reminders
            var maxReminders = Math.ceil(daydiff(seasonStart, seasonEnd) / interval); // maximum number of reminders per season

            var date = new Date();

            var id = 1;
            var usageNotifications = [];
            while (date < seasonEnd) 
            {
                // add the interval
                date.setDate(date.getDate() + interval);
                if (date < seasonStart) {
                    date = seasonStart;
                }

                usageNotifications.push({
                    id: id,
                    at: new Date(date.getTime()),
                    title: $translate.instant("Ski season is not over"),
                    text: $translate.instant("Make your picks and win!"),
                    sound: deviceIsAndroid ? 'file://resources/SecondGlance.mp3' : 'file://resources/SecondGlance.m4r'
                });
                // increment the id
                id++;
            }
            
            for (id; id <= maxReminders; id++) {
                _cancel(NTYPE.USAGE, id);
            }
            if (usageNotifications.length > 0) {
                // make sure this is only adding the monthly notifications
                if (usageNotifications.length <= 12) {
                    _schedule(NTYPE.USAGE, usageNotifications);
                }
            }
        },
        _updateRaceNotifications = function (userRaces) {
            var raceNotifications = [];
            userRaces.forEach(function (userRace) {
            	var race = userRace.race;

            	if (!race.completed) {
            		if (userRace.pickCount >= 10) {
            		    _cancel(NTYPE.RACE, race.id);
            		}
            		else {
            			// typically a notification should be one day before the race
            			var date = new Date(race.race_date);
            			date.setDate(date.getDate() - 1);
            			// however, in the case that the user opened the app by clicking on a notification,
            			// and is ready to starting picking the races that need to be picked,
            			// don't send a notification AGAIN and annoy them.  give them 15 minutes to pick before re-sending
            			var now = new Date();
            			if (date < now) {
            				date = now;
            				date.setMinutes(date.getMinutes() + 15);
            			}

            			raceNotifications.push({
            				id: race.id,
            				title: race.name,
            				at: date,
            				text: $translate.instant("The race starts soon and you haven't made your picks."),
            				sound: deviceIsAndroid ? 'file://resources/SecondGlance.mp3' : 'file://resources/SecondGlance.m4r'
            			});
            		}
            	}
            });
            if (raceNotifications.length > 0) {
                _schedule(NTYPE.RACE, raceNotifications);
            }
        },
        _updateChallengeNotifications = function (challenges) {
            var challengeNotifications = [];
            challenges.forEach(function (challenge) {

                var date = new Date(challenge.end_date);
                if (date > new Date()) {
                    if (!challenge.joined) {
                        _cancel(NTYPE.CHALLENGE, challenge.id);
                    }
                    else {
                        challengeNotifications.push({
                            id: challenge.id,
                            title: challenge.name,
                            at: date,
                            text: $translate.instant("Did you complete the challenge?"),
                            sound: deviceIsAndroid ? 'file://resources/Concern.mp3' : 'file://resources/Concern.m4r'
                        });
                    }
                }
            });
            if (challengeNotifications.length > 0) {
                _schedule(NTYPE.CHALLENGE, challengeNotifications);
            }
        };

    //----------------

    //----Public----
    return {
        Init: _init,
        UpdateRaceNotifications: _updateRaceNotifications,
        UpdateChallengeNotifications: _updateChallengeNotifications
    };
    //-------------
}]);
